//
//  iNetwork.h
//  iNetwork
//
//  Created by b4j on 9/9/14.
//  Copyright (c) 2014 Anywhere Software. All rights reserved.
//

#import <Foundation/Foundation.h>

@class B4I;
@class B4IInputStream;
@class B4IOutputStream;
@class B4IFastSocket;
@class B4IArray;
@class B4IUDPPacket;

//~hide
@interface FastSocketOutputStream :NSOutputStream
- (instancetype)init:(B4IFastSocket *)s;
@end
//~version:1.32
//~shortname:Socket
//~event:Connected (Successful As Boolean)
@interface B4ISocketWrapper : NSObject
/**
* Gets or sets the socket timeout, measured in milliseconds.
*By default there is no timeout.
*/
@property (nonatomic)int Timeout;
/**
* Gets the network input stream. In most cases the stream should be passed to an AsyncStreams object.
*/
@property (nonatomic, readonly)B4IInputStream *InputStream;
/**
* Gets the network output stream. In most cases the stream should be passed to an AsyncStreams object.
*/
@property (nonatomic, readonly)B4IOutputStream *OutputStream;
/**
* Returns true if the socket is connected.
*/
@property (nonatomic, readonly)BOOL Connected;
//~hide
@property (nonatomic) B4IFastSocket *socket;
- (void)Initialize:(NSString *)EventName;

- (BOOL)IsInitialized;

/**
* Tries to to connect to the given address. The Connected event will be raised when operation completes.
*Host - The host name or ip address.
* Port - Port number.
* Timeout - Connection timeout measured in milliseconds. Pass 0 to disable the timeout.
*/
- (void)Connect:(B4I *)bi :(NSString *)Host :(int)Port :(int)Timeout;

/**
* Closes the socket and the streams. It is safe to call this method multiple times.
*/
- (void)Close;
@end
//~shortname:ServerSocket
//~event:NewConnection (Successful As Boolean, NewSocket As Socket)
@interface B4IServerSocketWrapper : NSObject

+ (NSString *)GetBroadcastAddress;

/**
* Initializes the server socket and binds it to the given port.
*/
- (void)Initialize:(B4I *)bi1 :(int)Port :(NSString *)EventName;

- (BOOL)IsInitialized;

/**
* Returns the device IP address. Preferably the wifi address. Returns 127.0.0.1 if there is no ip address.
*/
- (NSString *)GetMyIP;

/**
* Returns the wifi address. Returns 127.0.0.1 if there is no ip address.
*/
- (NSString *)GetMyWifiIp;

/**
* Starts listening for incoming connections. The NewConnection event will be raised when there is a new connection.
*Calling Listen when the server is already listening will not do anything.
*/
- (void)Listen;

/**
* Closes the ServerSocket.
*/
- (void)Close;
@end
/**
* UDPSocket supports sending and receiving UDPPackets.
*/
//~shortname:UDPSocket
//~Event:PacketArrived(Packet As UDPPacket)
@interface B4IUDPSocket : NSObject
/**
* Initializes the socket and starts listening for packets.
* EventName - Sets the sub that will handle the PacketArrived event.
* Port - Local port to listen on. Pass 0 to let the OS choose an available port.
* ReceiveBufferSize - The maximum size of received packets. Pass 0 if the socket should not listen to incoming packets.
*/
- (void)Initialize:(B4I *)bi :(NSString *)EventName :(int)Port :(int)ReceiveBufferSize;

- (BOOL)IsInitialized;
/**
 * Returns the network broadcast address.
 */
- (NSString *)GetBroadcastAddress;
/**
* Sends the packet.
*/
- (void)Send:(B4IUDPPacket *)Packet;

- (void)SendBroadcast:(B4IUDPPacket *)Packet;

/**
* Closes the socket/
*/
- (void)Close;
@end
//~shortname:UDPPacket
@interface B4IUDPPacket : NSObject
//Gets the data length.
@property (nonatomic, readonly)int Length;
//Gets the data.
@property (nonatomic, readonly)B4IArray * /*byte,1*/ Data;
//Gets the received data offset.
@property (nonatomic, readonly)int Offset;
//Gets the port of the other socket.
@property (nonatomic, readonly)int Port;
//Gets the IP address of the other socket.
@property (nonatomic, readonly)NSString *HostAddress;
/**
* Initializes the packet with the given data and destination.
* You can reuse the buffer after this call (data is copied).
 */
- (void)Initialize:(B4IArray */*byte,1*/)Data :(NSString *)Host :(int)Port;

/**
* Same as Initialize. The data is sent based on the given offset and length.
*/
- (void)Initialize2:(B4IArray */*byte,1*/)Data :(int)Offset :(int)Length :(NSString *)Host :(int)Port;

//~hide
- (instancetype)initWithLength:(int)Length Offset:(int)Offset Port:(int)Port HostAddress:(NSString *)HostAddress Data:(B4IArray */*byte,1*/)Data;
@end